@@ -8,20 +8,24 @@ from time import mktime
 
 import six
 from django.conf import settings
-from django.core.mail.message import sanitize_address, DEFAULT_ATTACHMENT_MIME_TYPE
-from django.utils.encoding import force_text
+from django.core.mail.message import DEFAULT_ATTACHMENT_MIME_TYPE, sanitize_address
 from django.utils.functional import Promise
-from django.utils.timezone import utc, get_fixed_timezone
+from django.utils.timezone import get_fixed_timezone, utc
 from requests.structures import CaseInsensitiveDict
 from six.moves.urllib.parse import urlsplit, urlunsplit
 
+from .exceptions import AnymailConfigurationError, AnymailInvalidAddress
+
+if six.PY2:
+    from django.utils.encoding import force_text as force_str
+else:
+    from django.utils.encoding import force_str
+
 try:
     from collections.abc import Mapping, MutableMapping  # Python 3.3+
 except ImportError:
     from collections import Mapping, MutableMapping
 
-from .exceptions import AnymailConfigurationError, AnymailInvalidAddress
-
 
 BASIC_NUMERIC_TYPES = six.integer_types + (float,)  # int, float, and (on Python 2) long
 
@@ -147,7 +151,7 @@ def parse_address_list(address_list, field=None):
     # from the list -- which may split comma-seperated strings into multiple addresses.
     # (See django.core.mail.message: EmailMessage.message to/cc/bcc/reply_to handling;
     # also logic for ADDRESS_HEADERS in forbid_multi_line_headers.)
-    address_list_strings = [force_text(address) for address in address_list]  # resolve lazy strings
+    address_list_strings = [force_str(address) for address in address_list]  # resolve lazy strings
     name_email_pairs = getaddresses(address_list_strings)
     if name_email_pairs == [] and address_list_strings == [""]:
         name_email_pairs = [('', '')]  # getaddresses ignores a single empty string
